@@ -60,6 +60,9 @@ class FACTORY(object):
     def __call__(self):
         return self._callable()
 
+    def __repr__(self):
+        return 'FACTORY({0!r})'.format(self._callable)
+
 
 ########################################################################
 # Keep track of fields, both with and without defaults.
@@ -237,6 +240,27 @@ def _make_fn(name, chain_fn, args, defaults):
     return locals_[name]
 
 
+########################################################################
+# Produce a docstring for the class.
+
+def _field_name_with_default(name, default):
+    if default is NO_DEFAULT:
+        return name
+    return '{0}={1!r}'.format(name, default)
+
+def _build_docstring(typename, fields, defaults):
+    # We can use NO_DEFAULT as a sentinel here, becuase it will never be
+    #  present in defaults. By this point, it has been removed and replaced
+    #  with actual default values.
+
+    # The defaults make this a little tricky. Append a sentinel in
+    #  front of defaults until it's the same length as fields. The
+    #  sentinel value is used in _name_with_default
+    defaults = [NO_DEFAULT] * (len(fields) - len(defaults)) + defaults
+    return '{0}({1})'.format(typename, ', '.join(_field_name_with_default(name, default)
+                                                 for name, default in zip(fields, defaults)))
+
+
 ########################################################################
 # The actual namedlist factory function. Needs a docstring.
 def namedlist(typename, field_names, default=NO_DEFAULT, rename=False,
@@ -300,6 +324,7 @@ def namedlist(typename, field_names, default=NO_DEFAULT, rename=False,
                  '__setitem__': _setitem,
                  '__iter__': _iter,
                  '__hash__': None,
+                 '__doc__': _build_docstring(typename, all_field_names, defaults),
                  'count': _count,
                  'index': _index,
                  '_asdict': _asdict,
@@ -521,6 +546,7 @@ if __name__ == '__main__':
         def test_fields(self):
             Point = namedlist('Point', 'x y z')
             self.assertEqual(Point._fields, ('x', 'y', 'z'))
+            self.assertIsInstance(Point._fields, tuple)
 
             Point = namedlist('Point', 'x y z', 100)
             self.assertEqual(Point._fields, ('x', 'y', 'z'))
@@ -528,6 +554,10 @@ if __name__ == '__main__':
             Point = namedlist('Point', [('x', 0), ('y', 0), ('z', 0)])
             self.assertEqual(Point._fields, ('x', 'y', 'z'))
 
+            Point = namedlist('Point', '')
+            self.assertEqual(Point._fields, ())
+            self.assertIsInstance(Point._fields, tuple)
+
         def test_pickle(self):
             for p in (TestRT0(), TestRT(x=10, y=20, z=30)):
                 for module in pickle_modules:
@@ -656,6 +686,37 @@ if __name__ == '__main__':
             b = A()
             self.assertEqual(b.x, [])
 
+        def test_factory_for_default(self):
+            # make sure FACTORY works for the global default
+            A = namedlist('A', 'x y', default=FACTORY(list))
+            a = A()
+            self.assertEqual(a.x, [])
+            self.assertEqual(a.y, [])
+
+            a.x.append(4)
+            self.assertEqual(a.x, [4])
+            a.y.append(4)
+            self.assertEqual(a.y, [4])
+
+            b = A()
+            self.assertEqual(b.x, [])
+            self.assertEqual(b.y, [])
+
+            # mix and match FACTORY and a non-callable mutable default
+            A = namedlist('A', [('x', []), 'y'], default=FACTORY(list))
+            a = A()
+            self.assertEqual(a.x, [])
+            self.assertEqual(a.y, [])
+
+            a.x.append(4)
+            self.assertEqual(a.x, [4])
+            a.y.append(4)
+            self.assertEqual(a.y, [4])
+
+            b = A()
+            self.assertEqual(b.x, [4])
+            self.assertEqual(b.y, [])
+
         def test_unhashable(self):
             Point = namedlist('Point', 'a b')
             p = Point(1, 2)
@@ -706,5 +767,37 @@ if __name__ == '__main__':
             self.assertEqual(a.index(0, 5, 12), 5)
             self.assertRaises(ValueError, a.index, 0, 12)
 
+        def test_docstring(self):
+            Point = namedlist('Point', '')
+            self.assertEqual(Point.__doc__, 'Point()')
+
+            Point = namedlist('Point', 'dx')
+            self.assertEqual(Point.__doc__, 'Point(dx)')
+
+            Point = namedlist('Point', 'x')
+            self.assertEqual(Point.__doc__, 'Point(x)')
+
+            Point = namedlist('Point', 'dx dy, dz')
+            self.assertEqual(Point.__doc__, 'Point(dx, dy, dz)')
+
+            Point = namedlist('Point', 'dx dy dz', default=10)
+            self.assertEqual(Point.__doc__, 'Point(dx=10, dy=10, dz=10)')
+
+            Point = namedlist('Point', 'dx, dy, dz', default=FACTORY(10))
+            self.assertEqual(Point.__doc__, 'Point(dx=FACTORY(10), dy=FACTORY(10), dz=FACTORY(10))')
+
+            Point = namedlist('Point', ['dx', 'dy', ('dz', 11.0)], default=10)
+            self.assertEqual(Point.__doc__, 'Point(dx=10, dy=10, dz=11.0)')
+
+            Point = namedlist('Point', ['dx', 'dy', ('dz', 11.0)], default=FACTORY(list))
+            if _PY2:
+                list_repr = "<type 'list'>"
+            else:
+                list_repr = "<class 'list'>"
+            self.assertEqual(Point.__doc__, "Point(dx=FACTORY({0}), dy=FACTORY({0}), dz=11.0)".format(list_repr))
+
+            Point = namedlist('Point', ['dx', 'dy', ('dz', FACTORY(11.0))], default=[])
+            self.assertEqual(Point.__doc__, 'Point(dx=[], dy=[], dz=FACTORY(11.0))')
+
 
     unittest.main()
